@@ -29,13 +29,13 @@
 #include <config.h>
 #include "eel-dnd.h"
 
-#include <ctype.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkseparatormenuitem.h>
 #include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-popup-menu.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomevfs/gnome-vfs-find-directory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -372,10 +372,10 @@ is_path_that_gnome_uri_list_extract_filenames_can_parse (const char *path)
 	/* It strips leading and trailing spaces. So it can't handle
 	 * file names with leading and trailing spaces.
 	 */
-	if (isspace ((guchar) path [0])) {
+	if (g_ascii_isspace (path [0])) {
 		return FALSE;
 	}
-	if (isspace ((guchar) path [strlen (path) - 1])) {
+	if (g_ascii_isspace (path [strlen (path) - 1])) {
 		return FALSE;
 	}
 
@@ -489,52 +489,120 @@ eel_drag_modifier_based_action (int default_action, int non_default_action)
 	return default_action;
 }
 
-/* The menu of DnD actions */
-static GnomeUIInfo menu_items[] = {
-	GNOMEUIINFO_ITEM_NONE (N_("_Move here"), NULL, NULL),
-	GNOMEUIINFO_ITEM_NONE (N_("_Copy here"), NULL, NULL),
-	GNOMEUIINFO_ITEM_NONE (N_("_Link here"), NULL, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_ITEM_NONE (N_("Cancel"), NULL, NULL),
-	GNOMEUIINFO_END
-};
+typedef struct
+{
+	GMainLoop *loop;
+	GdkDragAction chosen;
+} DropActionMenuData;
+
+static void
+menu_deactivate_callback (GtkWidget *menu,
+			  gpointer   data)
+{
+	DropActionMenuData *damd;
+	
+	damd = data;
+
+	if (g_main_loop_is_running (damd->loop))
+		g_main_loop_quit (damd->loop);
+}
+
+static void
+drop_action_activated_callback (GtkWidget  *menu_item,
+				gpointer    data)
+{
+	DropActionMenuData *damd;
+	
+	damd = data;
+
+	damd->chosen = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item),
+							   "action"));
+
+	if (g_main_loop_is_running (damd->loop))
+		g_main_loop_quit (damd->loop);
+}
+
+static void
+append_drop_action_menu_item (GtkWidget          *menu,
+			      const char         *text,
+			      GdkDragAction       action,
+			      gboolean            sensitive,
+			      DropActionMenuData *damd)
+{
+	GtkWidget *menu_item;
+
+	menu_item = gtk_menu_item_new_with_mnemonic (text);
+	gtk_widget_set_sensitive (menu_item, sensitive);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	g_object_set_data (G_OBJECT (menu_item),
+			   "action",
+			   GINT_TO_POINTER (action));
+	
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (drop_action_activated_callback),
+			  damd);
+
+	gtk_widget_show (menu_item);
+}
 
 /* Pops up a menu of actions to perform on dropped files */
 GdkDragAction
 eel_drag_drop_action_ask (GdkDragAction actions)
 {
 	GtkWidget *menu;
-	int action;
-	char *saved_domain;
-
+	GtkWidget *menu_item;
+	DropActionMenuData damd;
+	
 	/* Create the menu and set the sensitivity of the items based on the
 	 * allowed actions.
 	 */
-	saved_domain = textdomain (GNOME_EXPLICIT_TRANSLATION_DOMAIN);
-	menu = gnome_popup_menu_new (menu_items);
-	textdomain (saved_domain);
+	menu = gtk_menu_new ();
+	
+	append_drop_action_menu_item (menu, _("_Move here"),
+				      GDK_ACTION_MOVE,
+				      (actions & GDK_ACTION_MOVE) != 0,
+				      &damd);
 
-	gtk_widget_set_sensitive (menu_items[0].widget, (actions & GDK_ACTION_MOVE) != 0);
-	gtk_widget_set_sensitive (menu_items[1].widget, (actions & GDK_ACTION_COPY) != 0);
-	gtk_widget_set_sensitive (menu_items[2].widget, (actions & GDK_ACTION_LINK) != 0);
+	append_drop_action_menu_item (menu, _("_Copy here"),
+				      GDK_ACTION_COPY,
+				      (actions & GDK_ACTION_COPY) != 0,
+				      &damd);
+	
+	append_drop_action_menu_item (menu, _("_Link here"),
+				      GDK_ACTION_LINK,
+				      (actions & GDK_ACTION_LINK) != 0,
+				      &damd);
 
-	switch (gnome_popup_menu_do_popup_modal (menu, NULL, NULL, NULL, NULL)) {
-	case 0:
-		action = GDK_ACTION_MOVE;
-		break;
-	case 1:
-		action = GDK_ACTION_COPY;
-		break;
-	case 2:
-		action = GDK_ACTION_LINK;
-		break;
-	default:
-		action = 0; 
-	}
+	menu_item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+	
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Cancel"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+	
+	damd.chosen = 0;
+	damd.loop = g_main_loop_new (NULL, FALSE);
 
-	gtk_widget_destroy (menu);
+	g_signal_connect (menu, "deactivate",
+			  G_CALLBACK (menu_deactivate_callback),
+			  &damd);
+	
+	gtk_grab_add (menu);
+	
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+			NULL, NULL, 0, GDK_CURRENT_TIME);
+	
+	g_main_loop_run (damd.loop);
 
-	return action;
+	gtk_grab_remove (menu);
+	
+	g_main_loop_unref (damd.loop);
+	
+	gtk_object_sink (GTK_OBJECT (menu));
+	
+	return damd.chosen;
 }
 
 gboolean
