@@ -23,9 +23,9 @@
 */
 
 #include <config.h>
-
 #include "eel-enumeration.h"
 
+#include "eel-debug.h"
 #include "eel-glib-extensions.h"
 #include "eel-lib-self-check-functions.h"
 #include "eel-string.h"
@@ -398,7 +398,7 @@ enumeration_table_get (void)
 
 	enumeration_table = g_hash_table_new (g_str_hash, g_str_equal);
 
-	g_atexit (enumeration_table_free);
+	eel_debug_call_at_shutdown (enumeration_table_free);
 
 	return enumeration_table;
 }
