@@ -26,24 +26,26 @@
 #include <config.h>
 #include "eel-gdk-pixbuf-extensions.h"
 
+#include "eel-art-gtk-extensions.h"
+#include "eel-debug-drawing.h"
+#include "eel-debug.h"
 #include "eel-gdk-extensions.h"
 #include "eel-glib-extensions.h"
-#include "eel-string.h"
-#include "eel-art-gtk-extensions.h"
 #include "eel-lib-self-check-functions.h"
-#include "eel-debug-drawing.h"
+#include "eel-string.h"
 #include <gdk-pixbuf/gdk-pixbuf-loader.h>
-#include <gdk/gdkx.h>
 #include <gdk/gdkprivate.h>
+#include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <math.h>
+#include <pango/pangoft2.h>
 #include <png.h>
 
 #define LOAD_BUFFER_SIZE 65536
 
-const ArtIRect eel_gdk_pixbuf_whole_pixbuf = { -1, -1, -1, -1 };
+const ArtIRect eel_gdk_pixbuf_whole_pixbuf = { G_MININT, G_MININT, G_MAXINT, G_MAXINT };
 
 struct EelPixbufLoadHandle {
 	GnomeVFSAsyncHandle *vfs_handle;
@@ -126,32 +128,34 @@ eel_gdk_pixbuf_load (const char *uri)
 		}
 		if (!gdk_pixbuf_loader_write (loader,
 					      buffer,
-					      bytes_read)) {
+					      bytes_read,
+					      NULL)) {
 			result = GNOME_VFS_ERROR_WRONG_FORMAT;
 			break;
 		}
 	}
 
 	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-		gtk_object_unref (GTK_OBJECT (loader));
+		g_object_unref (loader);
 		gnome_vfs_close (handle);
 		return NULL;
 	}
 
 	gnome_vfs_close (handle);
-	gdk_pixbuf_loader_close (loader);
+	gdk_pixbuf_loader_close (loader, NULL);
 
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 	if (pixbuf != NULL) {
-		gdk_pixbuf_ref (pixbuf);
+		g_object_ref (pixbuf);
 	}
-	gtk_object_unref (GTK_OBJECT (loader));
+	g_object_unref (loader);
 
 	return pixbuf;
 }
 
 EelPixbufLoadHandle *
 eel_gdk_pixbuf_load_async (const char *uri,
+			   int priority,
 			   EelPixbufLoadCallback callback,
 			   gpointer callback_data)
 {
@@ -164,6 +168,7 @@ eel_gdk_pixbuf_load_async (const char *uri,
 	gnome_vfs_async_open (&handle->vfs_handle,
 			      uri,
 			      GNOME_VFS_OPEN_READ,
+			      priority,
 			      file_opened_callback,
 			      handle);
 
@@ -212,7 +217,8 @@ file_read_callback (GnomeVFSAsyncHandle *vfs_handle,
 	if (result == GNOME_VFS_OK && bytes_read != 0) {
 		if (!gdk_pixbuf_loader_write (handle->loader,
 					      buffer,
-					      bytes_read)) {
+					      bytes_read,
+					      NULL)) {
 			result = GNOME_VFS_ERROR_WRONG_FORMAT;
 		}
 		gnome_vfs_async_read (handle->vfs_handle,
@@ -238,7 +244,7 @@ static void
 free_pixbuf_load_handle (EelPixbufLoadHandle *handle)
 {
 	if (handle->loader != NULL) {
-		gtk_object_unref (GTK_OBJECT (handle->loader));
+		g_object_unref (handle->loader);
 	}
 	g_free (handle);
 }
@@ -249,7 +255,7 @@ load_done (EelPixbufLoadHandle *handle, GnomeVFSResult result, gboolean get_pixb
 	GdkPixbuf *pixbuf;
 
 	if (handle->loader != NULL) {
-		gdk_pixbuf_loader_close (handle->loader);
+		gdk_pixbuf_loader_close (handle->loader, NULL);
 	}
 
 	pixbuf = get_pixbuf ? gdk_pixbuf_loader_get_pixbuf (handle->loader) : NULL;
@@ -277,7 +283,7 @@ eel_cancel_gdk_pixbuf_load (EelPixbufLoadHandle *handle)
 
 /* return the average value of each component */
 guint32
-eel_gdk_pixbuf_average_value_argb (GdkPixbuf *pixbuf)
+eel_gdk_pixbuf_average_value (GdkPixbuf *pixbuf)
 {
 	guint64 a_total, r_total, g_total, b_total;
 	int row, column;
@@ -322,18 +328,6 @@ eel_gdk_pixbuf_average_value_argb (GdkPixbuf *pixbuf)
 		| ((b_total + dividend / 2) / dividend);
 }
 
-void
-eel_gdk_pixbuf_average_value (GdkPixbuf *pixbuf, GdkColor *average_color)
-{
-	guint32 argb;
-
-	argb = eel_gdk_pixbuf_average_value_argb (pixbuf);
-	average_color->pixel = 0;
-	average_color->red = ((argb >> 16) & 0xFF) * 0x101;
-	average_color->green = ((argb >> 8) & 0xFF) * 0x101;
-	average_color->blue = (argb & 0xFF) * 0x101;
-}
-
 double
 eel_gdk_scale_to_fit_factor (int width, int height,
 			     int max_width, int max_height,
@@ -438,8 +432,6 @@ eel_gdk_pixbuf_get_dimensions (const GdkPixbuf *pixbuf)
  * @color: The color to use.
  *
  * Fill the rectangle with the the given color.
- * Use the given rectangle if not NULL.
- * If rectangle is NULL, fill the whole pixbuf.
  */
 void
 eel_gdk_pixbuf_fill_rectangle_with_color (GdkPixbuf *pixbuf,
@@ -495,121 +487,19 @@ eel_gdk_pixbuf_fill_rectangle_with_color (GdkPixbuf *pixbuf,
 	}
 }
 
-/* utility routine for saving a pixbuf to a png file.
- * This was adapted from Iain Holmes' code in gnome-iconedit, and probably
- * should be in a utility library, possibly in gdk-pixbuf itself.
- */
 gboolean
 eel_gdk_pixbuf_save_to_file (const GdkPixbuf *pixbuf,
 			     const char *file_name)
 {
-	FILE *handle;
-  	char *buffer;
-	gboolean has_alpha;
-	int width, height, depth, rowstride;
-  	guchar *pixels;
-  	png_structp png_ptr;
-  	png_infop info_ptr;
-  	png_text text[2];
-  	int i;
-
-	g_return_val_if_fail (pixbuf != NULL, FALSE);
-	g_return_val_if_fail (file_name != NULL, FALSE);
-	g_return_val_if_fail (file_name[0] != '\0', FALSE);
-
-        handle = fopen (file_name, "wb");
-        if (handle == NULL) {
-        	return FALSE;
-	}
-
-	png_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-	if (png_ptr == NULL) {
-		fclose (handle);
-		return FALSE;
-	}
-
-	info_ptr = png_create_info_struct (png_ptr);
-	if (info_ptr == NULL) {
-		png_destroy_write_struct (&png_ptr, (png_infopp)NULL);
-		fclose (handle);
-	    	return FALSE;
-	}
-
-	if (setjmp (png_ptr->jmpbuf)) {
-		png_destroy_write_struct (&png_ptr, &info_ptr);
-		fclose (handle);
-		return FALSE;
-	}
-
-	png_init_io (png_ptr, handle);
-
-        has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
-	depth = gdk_pixbuf_get_bits_per_sample (pixbuf);
-	pixels = gdk_pixbuf_get_pixels (pixbuf);
-	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-
-	png_set_IHDR (png_ptr, info_ptr, width, height,
-		      depth, PNG_COLOR_TYPE_RGB_ALPHA,
-		      PNG_INTERLACE_NONE,
-		      PNG_COMPRESSION_TYPE_DEFAULT,
-		      PNG_FILTER_TYPE_DEFAULT);
-
-	/* Some text to go with the png image */
-	text[0].key = "Title";
-	text[0].text = (char *) file_name;
-	text[0].compression = PNG_TEXT_COMPRESSION_NONE;
-	text[1].key = "Software";
-	text[1].text = "Eel Thumbnail";
-	text[1].compression = PNG_TEXT_COMPRESSION_NONE;
-	png_set_text (png_ptr, info_ptr, text, 2);
-
-	/* Write header data */
-	png_write_info (png_ptr, info_ptr);
-
-	/* if there is no alpha in the data, allocate buffer to expand into */
-	if (has_alpha) {
-		buffer = NULL;
-	} else {
-		buffer = g_malloc(4 * width);
-	}
-	
-	/* pump the raster data into libpng, one scan line at a time */	
-	for (i = 0; i < height; i++) {
-		if (has_alpha) {
-			png_bytep row_pointer = pixels;
-			png_write_row (png_ptr, row_pointer);
-		} else {
-			/* expand RGB to RGBA using an opaque alpha value */
-			int x;
-			char *buffer_ptr = buffer;
-			char *source_ptr = pixels;
-			for (x = 0; x < width; x++) {
-				*buffer_ptr++ = *source_ptr++;
-				*buffer_ptr++ = *source_ptr++;
-				*buffer_ptr++ = *source_ptr++;
-				*buffer_ptr++ = 255;
-			}
-			png_write_row (png_ptr, (png_bytep) buffer);		
-		}
-		pixels += rowstride;
-	}
-	
-	png_write_end (png_ptr, info_ptr);
-	png_destroy_write_struct (&png_ptr, &info_ptr);
-	
-	g_free (buffer);
-		
-	fclose (handle);
-	return TRUE;
+	return gdk_pixbuf_save ((GdkPixbuf *) pixbuf,
+				file_name, "png", NULL, NULL);
 }
 
 void
 eel_gdk_pixbuf_ref_if_not_null (GdkPixbuf *pixbuf_or_null)
 {
 	if (pixbuf_or_null != NULL) {
-		gdk_pixbuf_ref (pixbuf_or_null);
+		g_object_ref (pixbuf_or_null);
 	}
 }
 
@@ -617,7 +507,7 @@ void
 eel_gdk_pixbuf_unref_if_not_null (GdkPixbuf *pixbuf_or_null)
 {
 	if (pixbuf_or_null != NULL) {
-		gdk_pixbuf_unref (pixbuf_or_null);
+		g_object_unref (pixbuf_or_null);
 	}
 }
 
@@ -951,7 +841,7 @@ eel_gdk_pixbuf_new_from_pixbuf_sub_area (GdkPixbuf *pixbuf,
 
 	/* Since we are going to be sharing the given pixbuf's data, we need 
 	 * to ref it.  It will be unreffed in the destroy function above */
-	gdk_pixbuf_ref (pixbuf);
+	g_object_ref (pixbuf);
 
 	/* Compute the offset into the pixel data */
 	pixels = 
@@ -1324,7 +1214,7 @@ static void
 destroy_global_buffer (void)
 {
 	if (global_buffer != NULL) {
-		gdk_pixbuf_unref (global_buffer);
+		g_object_unref (global_buffer);
 		global_buffer = NULL;
 	}
 }
@@ -1344,7 +1234,7 @@ eel_gdk_pixbuf_get_global_buffer (int minimum_width,
 			return global_buffer;
 		}
 
-		destroy_global_buffer ();		
+		destroy_global_buffer ();
 	}
 
 	g_assert (global_buffer == NULL);
@@ -1355,129 +1245,21 @@ eel_gdk_pixbuf_get_global_buffer (int minimum_width,
 
 	if (at_exit_deallocator_installed == FALSE) {
 		at_exit_deallocator_installed = TRUE;
-		g_atexit (destroy_global_buffer);
+		eel_debug_call_at_shutdown (destroy_global_buffer);
 	}
 
 	return global_buffer;
 }
 
-/* Same as gdk_pixbuf_get_from_drawable() except it deals with 
- * race conditions and other evil things that can happen */
-GdkPixbuf *
-eel_gdk_pixbuf_get_from_window_safe (GdkWindow *window,
-				     int x,
-				     int y,
-				     int width,
-				     int height)
-{
-	GdkWindowPrivate *window_private;
-	GdkPixbuf *pixbuf;
-	int error_code;
-	GdkWindowType save_window_type;
-	GdkColormap *colormap;
-
-	g_return_val_if_fail (window != NULL, NULL);
-
-
-	/* Push an error handler so that we can catch
-	 * the very rare (but possible) case where 
-	 * the GetImage() request fails.  See HACK2 below
-	 * for a more complete excuse.
-	 */
-	gdk_error_trap_push ();
-
-	/* Save the window type and colormap.  Otherwise
-	 * GdkPixbuf will try to fetch them from a hacked
-	 * window.  See HACK1 below for reason why we
-	 * hack the private window contents.
-	 */
-	save_window_type = gdk_window_get_type (window);
-	colormap = gdk_window_get_colormap (window);
-
-	/* HACK1:
-	 *
-	 * This horrible thing we do here is needed to
-	 * prevent GdkPixbuf from doing geometry sanity
-	 * checks on the window.  By pretending it
-	 * is a Pixmap, we fool GdkPixbuf into not doing
-	 * these checks.  
-	 *
-	 * Why ? The sanity checks that GdkPixbuf does
-	 * are good most of the time, but not 100% of 
-	 * the time.  This is because there is no guarantee
-	 * that when the x server gets the GetImage request,
-	 * the geometry of that window is what GdkPixbuf
-	 * thought it was.
-	 *
-	 * For example, the window manager could have triggered
-	 * a sequence of events causing the window to resize.
-	 */
-	window_private = (GdkWindowPrivate*) window;
-	window_private->window_type = GDK_WINDOW_PIXMAP;
-
-	pixbuf = gdk_pixbuf_get_from_drawable (NULL,
-					       window,
-					       colormap,
-					       x,
-					       y,
-					       0,
-					       0,
-					       width,
-					       height);
-
-	/* Restore the window's guts */
-	window_private->window_type = save_window_type;
-		
-	/* HACK2:
-	 *
-	 * Now we pop the error handler and see whether an error
-	 * occured.
-	 * 
-	 * It is very rare that an error might occur.  The conditions
-	 * under which it might are:
-	 *
-	 * 1) Race condition as described above in HACK1
-	 *
-	 * 2) Bogus coordinates and/or dimensions given to
-	 *    the GetImage() request - which GdkPixbuf cant
-	 *    safely sanity check against.
-	 *
-	 * So, if we get an error, we simply drop this request on
-	 * the floor and return NULL.  The caller needs to deal with
-	 * the fact that their request couldnt be executed.  Most of
-	 * the time, all that is needed is to simply ignore it.
-	 */
-	error_code = gdk_error_trap_pop ();
-	
-	if (error_code != 0) {
-		/* HACK3:
-		 *
-		 * The magical number "8" is the minor x error request
-		 * code.  That is the only error we are expecting.
-		 *
-		 * Otherwise we still return NULL, but the caller
-		 * gets a critical.
-		 */
-		g_return_val_if_fail (error_code == 8, NULL);
-
-		return NULL;
-	}
-
-	return pixbuf;
-}
-
 /**
  * eel_gdk_pixbuf_intersect:
  * @pixbuf: A GdkPixbuf.
  * @pixbuf_x: X coordinate of pixbuf.
  * @pixbuf_y: Y coordinate of pixbuf.
- * @rectangle: An ArtIRect or eel_gdk_pixbuf_whole_pixbuf.
+ * @rectangle: An ArtIRect.
  *
  * Return value: The intersection of the pixbuf and the given rectangle.
  *
- * If &rectangle is eel_gdk_pixbuf_whole_pixbuf, then the resulting
- * rectangle is a rectangle at the given orign with the pixbuf's dimensions.
- *
  */
 ArtIRect
 eel_gdk_pixbuf_intersect (const GdkPixbuf *pixbuf,
@@ -1494,10 +1276,6 @@ eel_gdk_pixbuf_intersect (const GdkPixbuf *pixbuf,
 	dimensions = eel_gdk_pixbuf_get_dimensions (pixbuf);
 	bounds = eel_art_irect_assign_dimensions (pixbuf_x, pixbuf_y, dimensions);
 
-	if (eel_art_irect_equal (rectangle, eel_gdk_pixbuf_whole_pixbuf)) {
-		return bounds;
-	}
-
 	art_irect_intersect (&intersection, &rectangle, &bounds);
 
 	/* In theory, this is not needed because a rectangle is empty
@@ -1512,6 +1290,105 @@ eel_gdk_pixbuf_intersect (const GdkPixbuf *pixbuf,
 	return intersection;
 }
 
+/**
+ * eel_gdk_pixbuf_draw_layout
+ * @pixbuf:    the pixbuf on which to draw the layout
+ * @x:         the X position of the left of the layout (in pixels)
+ * @y:         the Y position of the top of the layout (in pixels)
+ * @color:     the color to use to draw the text, in "eel RGB" format
+ * @layout:    a #PangoLayout
+ *
+ * Render a #PangoLayout onto a #GdkPixbuf.
+ */
+void
+eel_gdk_pixbuf_draw_layout (GdkPixbuf *pixbuf,
+			    int x, int y,
+			    guint32 color,
+			    PangoLayout *layout)
+{
+	PangoRectangle ink_rect;
+	int pixbuf_height, pixbuf_width;
+	FT_Bitmap ink_bitmap;
+	GdkPixbuf *ink_pixbuf;
+	int bx, by;
+	const guint8 *ps;
+	guint8 *pd;
+	guint8 r, g, b;
+	
+	pango_layout_get_pixel_extents (layout, &ink_rect, NULL);
+
+	ink_rect.x += x;
+	ink_rect.y += y;
+
+	/* Do an intersect operation the long way.
+	 * Sad to write this all out like this.
+	 */
+
+	if (ink_rect.x < 0) {
+		ink_rect.width += ink_rect.x;
+		ink_rect.x = 0;
+	}
+
+	if (ink_rect.y < 0) {
+		ink_rect.height += ink_rect.y;
+		ink_rect.y = 0;
+	}
+
+	pixbuf_width = gdk_pixbuf_get_width (pixbuf);
+	if (ink_rect.x + ink_rect.width > pixbuf_width) {
+		ink_rect.width = pixbuf_width - ink_rect.x;
+		if (ink_rect.width <= 0) {
+			return;
+		}
+	}
+
+	pixbuf_height = gdk_pixbuf_get_height (pixbuf);
+	if (ink_rect.y + ink_rect.height > pixbuf_height) {
+		ink_rect.height = pixbuf_height - ink_rect.y;
+		if (ink_rect.height <= 0) {
+			return;
+		}
+	}
+
+	ink_bitmap.rows = ink_rect.height;
+	ink_bitmap.width = ink_rect.width;
+	ink_bitmap.pitch = (ink_bitmap.width + 3) & ~3;
+	ink_bitmap.buffer = g_malloc0 (ink_bitmap.rows * ink_bitmap.pitch);
+	ink_bitmap.num_grays = 0x100;
+	ink_bitmap.pixel_mode = ft_pixel_mode_grays;
+
+	pango_ft2_render_layout (&ink_bitmap, layout, x - ink_rect.x, y - ink_rect.y);
+
+	ink_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
+				     ink_bitmap.width, ink_bitmap.rows);
+
+	r = EEL_RGBA_COLOR_GET_R (color);
+	g = EEL_RGBA_COLOR_GET_G (color);
+	b = EEL_RGBA_COLOR_GET_B (color);
+	
+	for (by = 0; by < ink_bitmap.rows; by++) {
+	        pd = gdk_pixbuf_get_pixels (ink_pixbuf)
+			+ by * gdk_pixbuf_get_rowstride (ink_pixbuf);
+		ps = ink_bitmap.buffer + by * ink_bitmap.pitch;
+		for (bx = 0; bx < ink_bitmap.width; bx++) {
+			*pd++ = r;
+		        *pd++ = g;
+			*pd++ = b;
+			*pd++ = *ps++;
+		}
+	}
+
+	g_free (ink_bitmap.buffer);
+
+	gdk_pixbuf_composite (ink_pixbuf, pixbuf,
+			      ink_rect.x, ink_rect.y,
+			      ink_bitmap.width, ink_bitmap.rows,
+			      ink_rect.x, ink_rect.y,
+			      1, 1, GDK_INTERP_BILINEAR, 0xFF);
+
+	g_object_unref (ink_pixbuf);
+}
+
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 static char *
@@ -1573,8 +1450,8 @@ check_average_value (int width, int height, const char* fill)
 		pixels [2] += gray_tweak;
 	}
 
-	average = eel_gdk_pixbuf_average_value_argb (pixbuf);
-	gdk_pixbuf_unref (pixbuf);
+	average = eel_gdk_pixbuf_average_value (pixbuf);
+	g_object_unref (pixbuf);
 
 	return g_strdup_printf ("%02X,%02X,%02X,%02X",
 				(average >> 16) & 0xFF,
@@ -1634,7 +1511,7 @@ eel_self_check_gdk_pixbuf_extensions (void)
 	clip_area = eel_art_irect_assign (80, 0, 100, 100);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 80, 0, 100, 100);
 
-	gdk_pixbuf_unref (pixbuf);
+	g_object_unref (pixbuf);
 
 	/* No checks for empty pixbufs because GdkPixbuf doesn't seem to allow them. */
 	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "00,00,00"), "00,00,00,FF");
