@@ -28,15 +28,15 @@
 #include <config.h>
 #include "eel-preferences-box.h"
 
-#include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
 #include <gtk/gtkclist.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtknotebook.h>
+#include <gtk/gtkstock.h>
+#include <eel/eel-gtk-extensions.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-stock.h>
 
 #define NUM_CATEGORY_COLUMNS 1
 #define CATEGORY_COLUMN 0
@@ -59,11 +59,11 @@ struct EelPreferencesBoxDetails
 };
 
 /* EelPreferencesBoxClass methods */
-static void      eel_preferences_box_initialize_class   (EelPreferencesBoxClass *preferences_box_class);
-static void      eel_preferences_box_initialize         (EelPreferencesBox      *preferences_box);
+static void      eel_preferences_box_class_init   (EelPreferencesBoxClass *preferences_box_class);
+static void      eel_preferences_box_init         (EelPreferencesBox      *preferences_box);
 
-/* GtkObjectClass methods */
-static void      eel_preferences_box_destroy            (GtkObject              *object);
+/* GObjectClass methods */
+static void      eel_preferences_box_finalize           (GObject                *object);
 
 /* Misc private stuff */
 static void      preferences_box_category_list_recreate (EelPreferencesBox      *preferences_box);
@@ -86,38 +86,33 @@ static int       preferences_box_find_row               (GtkCList
 							 char                   *pane_name);
 static void      user_level_changed_callback            (gpointer                callback_data);
 
-EEL_DEFINE_CLASS_BOILERPLATE (EelPreferencesBox, eel_preferences_box, GTK_TYPE_HBOX)
+EEL_CLASS_BOILERPLATE (EelPreferencesBox, eel_preferences_box, GTK_TYPE_HBOX)
 
 /*
  * EelPreferencesBoxClass methods
  */
 static void
-eel_preferences_box_initialize_class (EelPreferencesBoxClass *preferences_box_class)
+eel_preferences_box_class_init (EelPreferencesBoxClass *preferences_box_class)
 {
-	GtkObjectClass *object_class;
-	
-	object_class = GTK_OBJECT_CLASS (preferences_box_class);
-
-	/* GtkObjectClass */
-	object_class->destroy = eel_preferences_box_destroy;
+	G_OBJECT_CLASS (preferences_box_class)->finalize = eel_preferences_box_finalize;
 }
 
 static void
-eel_preferences_box_initialize (EelPreferencesBox *preferences_box)
+eel_preferences_box_init (EelPreferencesBox *preferences_box)
 {
 	preferences_box->details = g_new0 (EelPreferencesBoxDetails, 1);
 
 	eel_preferences_add_callback_while_alive ("user_level",
 						  user_level_changed_callback,
 						  preferences_box,
-						  GTK_OBJECT (preferences_box));
+						  G_OBJECT (preferences_box));
 }
 
 /*
- * GtkObjectClass methods
+ * GObjectClass methods
  */
 static void
-eel_preferences_box_destroy (GtkObject *object)
+eel_preferences_box_finalize (GObject *object)
 {
 	EelPreferencesBox *preferences_box;
 	
@@ -144,8 +139,7 @@ eel_preferences_box_destroy (GtkObject *object)
 	g_free (preferences_box->details->selected_pane);
 	g_free (preferences_box->details);
 	
-	/* Chain destroy */
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 /*
@@ -172,8 +166,9 @@ preferences_box_select_pane (EelPreferencesBox *preferences_box,
 		
 		if (eel_str_is_equal (pane_name, info->pane_name)) {
  			gtk_widget_show (GTK_WIDGET (info->pane_widget));
- 			gtk_notebook_set_page (GTK_NOTEBOOK (preferences_box->details->pane_notebook), 
- 					       g_list_position (preferences_box->details->panes, pane_iterator));
+ 			gtk_notebook_set_current_page
+				(GTK_NOTEBOOK (preferences_box->details->pane_notebook), 
+				 g_list_position (preferences_box->details->panes, pane_iterator));
 
 			g_free (preferences_box->details->selected_pane);
 			preferences_box->details->selected_pane = g_strdup (pane_name);
@@ -325,9 +320,9 @@ eel_preferences_box_new (void)
 	preferences_box->details->category_list = gtk_clist_new (NUM_CATEGORY_COLUMNS);
 
 	preferences_box->details->select_row_signal_id =
-		gtk_signal_connect (GTK_OBJECT (preferences_box->details->category_list), 
+		g_signal_connect (preferences_box->details->category_list, 
 				    "select_row",
-				    GTK_SIGNAL_FUNC (category_list_select_row_callback),
+				    G_CALLBACK (category_list_select_row_callback),
 				    preferences_box);
 
 	gtk_clist_set_selection_mode (GTK_CLIST (preferences_box->details->category_list), 
@@ -544,33 +539,26 @@ GtkWidget *
 eel_preferences_dialog_new (const char *title,
 			    const EelPreferencesPaneDescription *panes)
 {
-	static const char *stock_buttons[] = {
-		GNOME_STOCK_BUTTON_OK,
-		NULL
-	};
 	GtkWidget *dialog;
 	GtkWidget *preference_box;
 	GtkWidget *vbox;
 
 	g_return_val_if_fail (title != NULL, NULL);
 	
-	dialog = gnome_dialog_newv (title, stock_buttons);
+	dialog = gtk_dialog_new_with_buttons (title, NULL, 0,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
+					      NULL);
 
 	/* Setup the dialog */
-	gtk_window_set_policy (GTK_WINDOW (dialog), 
-			       FALSE,	/* allow_shrink */
-			       TRUE,	/* allow_grow */
-			       FALSE);	/* auto_shrink */
+	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
 
   	gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
 	
-	gnome_dialog_set_default (GNOME_DIALOG(dialog), 0);
-
-	eel_gtk_window_set_up_close_accelerator (GTK_WINDOW (dialog));
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
 	preference_box = eel_preferences_box_new ();
 
-	vbox = GNOME_DIALOG (dialog)->vbox;
+	vbox = GTK_DIALOG (dialog)->vbox;
 	
 	gtk_box_set_spacing (GTK_BOX (vbox), 10);
 	
@@ -596,9 +584,9 @@ eel_preferences_dialog_get_box (const GtkWindow *dialog)
 	const GList *last_node;
 	const GtkBoxChild *box_child;
 
-	g_return_val_if_fail (GNOME_IS_DIALOG (dialog), NULL);
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
 
-	vbox = GNOME_DIALOG (dialog)->vbox;
+	vbox = GTK_DIALOG (dialog)->vbox;
 
 	g_return_val_if_fail (GTK_IS_VBOX (vbox), NULL);
 	last_node = g_list_last (GTK_BOX (vbox)->children);
