@@ -844,7 +844,7 @@ preferences_item_create_smooth_font (EelPreferencesItem *item)
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	child = eel_font_picker_new ();
+	child = eel_font_picker_new (NULL);
 	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
 	preferences_item_add_connection_child (item,
