@@ -99,7 +99,7 @@ eel_string_list_new_from_string (const char *string,
  *
  * @string_list: A EelStringList or NULL
  *
- * Return value: A newly allocated string list that is equal to @other.
+ * Return value: A newly allocated string list that is equal to @string_list.
  */
 EelStringList *
 eel_string_list_copy (const EelStringList *string_list)
@@ -115,6 +115,59 @@ eel_string_list_copy (const EelStringList *string_list)
 	return copy;
 }
 
+/**
+ * eel_string_list_new_from_string_array:
+ *
+ * @string_array: A NULL terminated string srray.
+ * @case_sensitive: Flag indicating whether the new string list is case sensitive.
+ *
+ * Return value: A newly allocated string list populated with the contents of the
+ * NULL terminated @string_array.
+ */
+EelStringList *
+eel_string_list_new_from_string_array (const char * const string_array[],
+				       gboolean case_sensitive)
+{
+	EelStringList *string_list;
+
+	if (string_array == NULL) {
+		return NULL;
+	}
+
+	string_list = eel_string_list_new (case_sensitive);
+
+	eel_string_list_assign_from_string_array (string_list, string_array);
+
+	return string_list;
+}
+
+/**
+ * eel_string_list_assign_from_string_array:
+ *
+ * @string_list: A EelStringList.
+ * @string_array: A NULL terminated string srray.
+ *
+ * Populate a string list with the contents of a NULL terminated string array.
+ */
+void
+eel_string_list_assign_from_string_array (EelStringList *string_list,
+					  const char * const string_array[])
+{
+	guint i;
+	
+	g_return_if_fail (string_list != NULL);
+
+	eel_string_list_clear (string_list);
+
+	if (string_array == NULL) {
+		return;
+	}
+
+	for (i = 0; string_array[i] != NULL; i++) {
+		eel_string_list_insert (string_list, string_array[i]);
+	}
+}
+
 /**
  * eel_string_list_new_from_g_slist:
  *
@@ -230,6 +283,34 @@ eel_string_list_insert (EelStringList *string_list,
 					       g_strdup (string));
 }
 
+void
+eel_string_list_prepend (EelStringList *string_list,
+			 const char *string)
+{
+	g_return_if_fail (string_list != NULL);
+	g_return_if_fail (string != NULL);
+
+	string_list->strings = g_slist_prepend (string_list->strings,
+						g_strdup (string));
+}
+
+void
+eel_string_list_insert_string_list (EelStringList *string_list,
+				    const EelStringList *other_string_list)
+{
+	const GSList *node;
+
+	g_return_if_fail (string_list != NULL);
+
+	if (other_string_list == NULL) {
+		return;
+	}
+
+	for (node = other_string_list->strings; node; node = node->next) {
+		eel_string_list_insert (string_list, node->data);
+	}
+}
+
 char *
 eel_string_list_nth (const EelStringList *string_list,
 		     guint n)
@@ -513,7 +594,7 @@ eel_string_list_get_index_for_string (const EelStringList *string_list,
  * eel_string_list_as_string
  *
  * @string_list: A EelStringList
- * @delimiter: The string to use a delimeter, can be NULL.
+ * @delimiter: The string to use a delimiter, can be NULL.
  * @num_strings: Number of strings to concatenate.  Must be between 0 and length.
  *
  * Return value: An newly allocated string concatenation of all the items in the list.
@@ -715,8 +796,8 @@ eel_string_list_for_each (const EelStringList *string_list,
 }
 
 void
-eel_string_list_append (EelStringList *string_list,
-			const EelStringList *append_string_list)
+eel_string_list_append_string_list (EelStringList *string_list,
+				    const EelStringList *append_string_list)
 {
 	const GSList *node;
 
@@ -740,6 +821,14 @@ str_is_equal (const char *a,
 	return case_sensitive ? eel_str_is_equal (a, b) : eel_istr_is_equal (a, b);
 }
 
+void
+eel_string_list_reverse (EelStringList *string_list)
+{
+	g_return_if_fail (string_list != NULL);
+	
+	string_list->strings = g_slist_reverse (string_list->strings);
+}
+
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 static gboolean
@@ -798,6 +887,40 @@ compare_number (gconstpointer string_a,
 	return 1;
 }
 
+static EelStringList *
+test_string_list_reverse (const char *tokens,
+			  const char *delimiter)
+{
+	EelStringList *result;
+
+	result = eel_string_list_new_from_tokens (tokens, delimiter, TRUE);
+
+	eel_string_list_reverse (result);
+
+	return result;
+}
+
+static EelStringList *
+test_new_from_string_array (const char *strings,
+			    const char *delimiter)
+{
+	EelStringList *result;
+	char **string_array;
+	
+ 	g_return_val_if_fail (delimiter != NULL, NULL);
+
+	if (strings == NULL) {
+		return eel_string_list_new (TRUE);
+	}
+
+	string_array = g_strsplit (strings, delimiter, -1);
+	result = eel_string_list_new_from_string_array ((const char * const *) string_array,
+							TRUE);
+	g_strfreev (string_array);
+	
+	return result;
+}
+
 void
 eel_self_check_string_list (void)
 {
@@ -1396,6 +1519,22 @@ eel_self_check_string_list (void)
 		
 		eel_g_list_free_deep (glist);
 	}
+
+	/* eel_string_list_reverse */
+	EEL_CHECK_STRING_LIST_RESULT (test_string_list_reverse (NULL, ","), "", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_string_list_reverse ("", ","), "", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_string_list_reverse ("foo", ","), "foo", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_string_list_reverse ("foo,bar", ","), "bar,foo", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_string_list_reverse ("1,2,3,4,5", ","), "5,4,3,2,1", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_string_list_reverse ("1", ","), "1", ",");
+
+	/* eel_string_list_new_from_string_array */
+	EEL_CHECK_STRING_LIST_RESULT (test_new_from_string_array (NULL, ","), "", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_new_from_string_array ("", ","), "", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_new_from_string_array ("1", ","), "1", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_new_from_string_array ("1,2", ","), "1,2", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_new_from_string_array ("1,2,3,4,5", ","), "1,2,3,4,5", ",");
+	EEL_CHECK_STRING_LIST_RESULT (test_new_from_string_array ("foo,bar,baz", ","), "foo,bar,baz", ",");
 }
 
 #endif /* !EEL_OMIT_SELF_CHECK */
