@@ -58,16 +58,15 @@ style_get_color (const GtkStyle *style,
 	g_return_val_if_fail (state <= GTK_STATE_INSENSITIVE, empty);
 
 	switch (n) {
-	case 0: return style->fg[state]; break;
-	case 1: return style->bg[state]; break;
-	case 2: return style->light[state]; break;
-	case 3: return style->dark[state]; break;
-	case 4: return style->mid[state]; break;
-	case 5: return style->text[state]; break;
-	case 6: return style->base[state]; break;
-	default:
+	case 0: return style->fg[state];
+	case 1: return style->bg[state];
+	case 2: return style->light[state];
+	case 3: return style->dark[state];
+	case 4: return style->mid[state];
+	case 5: return style->text[state];
+	case 6: return style->base[state];
 	}
-	g_assert_not_reached ();
+	g_return_val_if_reached (empty);
 	return empty;
 }
 
@@ -83,16 +82,15 @@ style_get_gc (const GtkStyle *style,
 	g_return_val_if_fail (state <= GTK_STATE_INSENSITIVE, NULL);
 
 	switch (n) {
-	case 0: return style->fg_gc[state]; break;
-	case 1: return style->bg_gc[state]; break;
-	case 2: return style->light_gc[state]; break;
-	case 3: return style->dark_gc[state]; break;
-	case 4: return style->mid_gc[state]; break;
-	case 5: return style->text_gc[state]; break;
-	case 6: return style->base_gc[state]; break;
-	default:
+	case 0: return style->fg_gc[state];
+	case 1: return style->bg_gc[state];
+	case 2: return style->light_gc[state];
+	case 3: return style->dark_gc[state];
+	case 4: return style->mid_gc[state];
+	case 5: return style->text_gc[state];
+	case 6: return style->base_gc[state];
 	}
-	g_assert_not_reached ();
+	g_return_val_if_reached (NULL);
 	return NULL;
 }
 
