@@ -140,33 +140,6 @@ test_pixbuf_new_named (const char *name, float scale)
 	return pixbuf;
 }
 
-GtkWidget *
-test_image_new (const char *pixbuf_name,
-		float scale,
-		gboolean with_background)
-{
-	GtkWidget *image;
-
-	if (with_background) {
-		image = eel_image_new_with_background (NULL);
-	} else {
-		image = eel_image_new (NULL);
-	}
-
-	if (pixbuf_name != NULL) {
-		GdkPixbuf *pixbuf;
-
-		pixbuf = test_pixbuf_new_named (pixbuf_name, scale);
-
-		if (pixbuf != NULL) {
-			eel_image_set_pixbuf (EEL_IMAGE (image), pixbuf);
-			g_object_unref (pixbuf);
-		}
-	}
-
-	return image;
-}
-
 // /* Preferences hacks */
 // void
 // test_text_caption_set_text_for_int_preferences (EelTextCaption *text_caption,
